
/***********************************************
*            ***  DECODER 4511  ***            *
*  Libreria per il controllo del decoder 4511  *
*                                              *
*   Firmware version: 2.0                      *
*   Autore: Battistella Mirco                  *
*   Data: 09/11/2013                           *
*   Piattaforma software: MPLAB X v1.80        *
*   Compilatore: XC8 v1.20                     *
*                                              *
***********************************************/


#include <xc.h>
#ifndef DECODER_HCF4511BE_H
#define	DECODER_HCF4511BE_H

//Definizione uscite di comando HCF4511BE (se non sono gi state definite)
#ifndef A
#define A PORTBbits.RB0 //Ingresso A HCF4511BE
#endif
#ifndef B
#define B PORTBbits.RB1 //Ingresso B HCF4511BE
#endif
#ifndef C
#define C PORTBbits.RB2 //Ingresso C HCF4511BE
#endif
#ifndef D
#define D PORTBbits.RB3 //Ingresso D HCF4511BE
#endif


//Definizione costanti abilitazione display (se non sono gi stat definite)
#ifndef disp1
#define disp1 PORTBbits.RB7 //Uscita abilitazione catodo display 1
#endif
#ifndef disp2
#define disp2 PORTBbits.RB6 //Uscita abilitazione catodo display 2
#endif
#ifndef disp3
#define disp3 PORTBbits.RB5 //Uscita abilitazione catodo display 3
#endif
#ifndef disp4
#define disp4 PORTBbits.RB4 //Uscita abilitazione catodo display 4
#endif

void visualizzaCifra(unsigned char cifra, unsigned char numDisplay)
//Questa funzione serve per inviare i comandi al 4511 affinch visualizzi il numero corretto sul relativo display
{
    //Disabilitazione catodi display (spegnimento di tutti i display)
    disp1=0;
    disp2=0;
    disp3=0;
    disp4=0;

    switch(cifra) //Selezione comandi CD4511 in base alla cifra da visualizzare (ricavati dal datasheet)
    {
        case 0: //Numero 0
            A=0; B=0; C=0; D=0; break; //Imposta ingressi CD4511 come: A=0 - B=0 - C=0 - D=0
        case 1:
            A=1; B=0; C=0; D=0; break;
        case 2:
            A=0; B=1; C=0; D=0; break;
        case 3:
            A=1; B=1; C=0; D=0; break;
        case 4:
            A=0; B=0; C=1; D=0; break;
        case 5:
            A=1; B=0; C=1; D=0; break;
        case 6:
            A=0; B=1; C=1; D=0; break;
        case 7:
            A=1; B=1; C=1; D=0; break;
        case 8:
            A=0; B=0; C=0; D=1; break;
        case 9:
            A=1; B=0; C=0; D=1; break;
        default:                        //Se la cifra non  compresa fra 0 e 9 (volont di non visualizzare nessun numero o errore software)
            A=0; B=1; C=0; D=1; break;  //Non visualizzare nessun numero
    }

    switch(numDisplay) //Selezione display da abilitare
    {
        case 1: //Numero 1
            disp1=1; break; //Attiva catodo display 1
        case 2:
            disp2=1; break;
        case 3:
            disp3=1; break;
        case 4:
            disp4=1; break;
        default:                                        //Se il numero del display non  compreso fra 1 e 4 (errore software)
            disp1=0; disp2=0; disp3=0; disp4=0; break;  //Spegni tutti i display
    }
    return;
}

void pausa()
//Delay eseguito con un ciclo for (equivalente alla durata di visualizzazione di ogni singola cifra del display)
{
    unsigned char conteggio; //Variabile per la ripetizione del ciclo
    
    for(conteggio=0;conteggio<100;conteggio++)  //Ciclo for che si ripete per 100 volte
    {
    }

    return;
}

void visualizzaSingleDigit(unsigned char cifra1, unsigned char cifra2, unsigned char cifra3, unsigned char cifra4)
//Funzione per la visualizzazione ciclica delle cifre di tutti e quattro i display
{
    visualizzaCifra(cifra1, 1); //Visualizza la prima cifra sul primo display
    pausa();
    visualizzaCifra(cifra2, 2); //Visualizza la seconda cifra sul secondo display
    pausa();
    visualizzaCifra(cifra3, 3); //Visualizza la terza cifra sul terzo display
    pausa();
    visualizzaCifra(cifra4, 4); //Visualizza la quarta cifra sul quarto display
    pausa();
    disp4=0; //Spegni il quarto display (tutti i display sono spenti per avere la stessa durata di accensione ed evitare che una cifra
             //risulti pi luminosa di un'altra)
    return;
}

void visualizzaOra(unsigned char hh, unsigned char mm)
//Funzione che effettua un ciclo di visualizzazione dell'ora corrente sui quattro display
{
    unsigned char cifra1=0; //Varibile con il valore da visualizzare nel primo display
    unsigned char cifra2=0; //Varibile con il valore da visualizzare nel secondo display
    unsigned char cifra3=0; //Varibile con il valore da visualizzare nel terzo display
    unsigned char cifra4=0; //Varibile con il valore da visualizzare nel quarto display
    
    cifra1=hh/10;  //Assegno alla prima variabile le decine delle ore da visualizzare nel primo display
    cifra2=hh%10;  //Assegno alla seconda variabile le unit delle ore da visualizzare nel secondo display
    cifra3=mm/10;  //Assegno alla terza variabile le decine dei minuti da visualizzare nel terzo display
    cifra4=mm%10;  //Assegno alla quarta variabile le unit dei minuti da visualizzare nel quarto display

    visualizzaSingleDigit(cifra1, cifra2, cifra3, cifra4); //Eseguo un ciclo di visualizzazione ciclico su tutti e quattro i display

    return;
}

#endif	/* DECODER_HCF4511BE_H */

