
/***********************************************
*                                              *
*           ***  OROLOGIO UP  ***              *
*                                              *
*     Con decoder 4511 e display a 4 cifre     *
*                                              *
*                                              *
*   Firmware version: 2.0                      *
*   autore: Battistella Mirco                  *
*   Data: 09/11/2013                           *
*   MCU: PIC16F1827                            *
*   Compilatore: XC8 v. 1.20                   *
*   Piattaforma software: MPLAB X 1.80         *
*                                              *
***********************************************/

//Inclusione file accessori principali
#include <xc.h>

//Definizione costanti numeriche uscite verso CD4511
#define A PORTBbits.RB0 //Ingresso A CD4511
#define B PORTBbits.RB1 //Ingresso B CD4511
#define C PORTBbits.RB2 //Ingresso C CD4511
#define D PORTBbits.RB3 //Ingresso D CD4511

//Definizione costanti abilitazione display
#define disp1 PORTBbits.RB7 //Uscita abilitazione catodo display 1
#define disp2 PORTBbits.RB6 //Uscita abilitazione catodo display 2
#define disp3 PORTBbits.RB5 //Uscita abilitazione catodo display 3
#define disp4 PORTBbits.RB4 //Uscita abilitazione catodo display 4

//Definizione costante punto decimale
#define dp PORTAbits.RA0  //Abilitazione punto decimale

//Definizione pulsanti regolazione
#define S1 PORTAbits.RA4  // H- / Attivazione programmazione / Conferma parametri
#define S2 PORTAbits.RA3  // H+
#define S3 PORTAbits.RA2  // M- / Decremento parametri
#define S4 PORTAbits.RA1  // M+ / Incremento parametri


//Variabili pubbliche
unsigned int ms=0; //Variabile per il conteggio dei millisecondi
unsigned char sec=0; //Variabile per il conteggio dei
unsigned char min=34; //Variabile per il conteggio dei
unsigned char ore=12; //Variabile per il conteggio delle ore
unsigned char sign=0; //Variabile del segno della calibrazione rispetto al clock
unsigned char calib1=0; //Calibrazione principale (precisione: 1ms)
unsigned char calib2=0; //Seconda calibrazione (precisione: 0.01ms)
unsigned char calib3=0; //Terza calibrazione (precisione: 0.0001ms)
unsigned char enFx=0; //Flag abilitazione effetti display
const unsigned char indSign=0; //Indirizzo in memoria del segno della calibrazione
const unsigned char indCalib1=1; //Indirizzo in memoria della calibrazione principale
const unsigned char indCalib2=2; //Indirizzo in memoria della seconda calibrazione
const unsigned char indCalib3=3; //Indirizzo in memoria della terza calibrazione
const unsigned char indEnFx=4; //Indirizzo in memoria dell'abilitazione degli effetti display
unsigned char calib2Count=0; //Contatore per l'attivazione della seconda calibrazione
unsigned char calib3Count=0; //Contatore per l'attivazione della terza calibrazione
unsigned char enDp=0; //Flag abilitazione del punto decimale
unsigned char clkEnFx=0; //Flag abilitazione effetti display ad ogni incremento dei minuti
unsigned char num[4]={0,0,0,0}; //Matrice numeri provvisori da caricare sui display (necessaria per la generazione degli effetti)
unsigned char numEffetto=0; //Contatore per la selezione casuale di un effetto display


//Inclusione file accessori personalizzati
#include "Settaggi.h" //File con i settaggi principali
#include "eepromX_PIC16F1827.h" //File per il dialogo con la memoria eeprom del pic
#include "Decoder_4511.h" //File per il comando del decoder 4511
#include "Effetti.h" //File per la generazione di effetti display
#include "Programmazione.h" // File per la programmazione dei parametri di calibrazione e e degli effetti dell'orologio

//Predichiarazione funzioni
void antirimbalzo();

//Configurazione fuses principali (effettuati con il Configuration Bits integrato)
// CONFIG1
#pragma config FOSC = HS        // Oscillator Selection (HS Oscillator, High-speed crystal/resonator connected between OSC1 and OSC2 pins)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = ON    // Clock Out Enable (CLKOUT function is enabled on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will not cause a Reset)
#pragma config BORV = 0        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

generaEffetto() //Funzione per la gestione del contatore necessario alla generazione di effetti casuali
{
    if(enFx==1) //Se gli effetti sono abilitati
    {
        effettoCasuale(ore, min, numEffetto); //Chiama la funzione per la generazione di effetti casuali

        if(numEffetto<30) //Se il contatore  minore di 30
        {
            numEffetto++; //Incrementa il contatore
        }
        else    //Altrimenti
        {
            numEffetto==0; //Azzera il contatore
        }
    }
}

//Routine principale
int main(void)
{
    settaggiPrincipali(); //Richiamo i settaggi principali

    sign=LeggiChar(indSign); //Leggo il segno della calibrazione dalla memoria
    calib1=LeggiChar(indCalib1); //Leggo il valore della calibrazione principale dalla memoria
    calib2=LeggiChar(indCalib2); //Leggo il valore della seconda calibrazione dalla memoria
    calib3=LeggiChar(indCalib3); //Leggo il valore della terza calibrazione dalla memoria
    enFx=LeggiChar(indEnFx); //Leggo il valore del flag di abilitazione degli effetti display dalla memoria

    if(sign==255 || calib1==255 || calib2==255 || calib3==255 || enFx==255) //Se uno o pi valori letti sono uguali a 255, significa che
    {                                                                       //si  verificato un errore o, semplicemente, che  il primo
                                                                            //avvio dell'orologio e i parametri non sonostati impostati.
        ScriviChar(0,indSign); //Quindi imposto a positivo il segno della calibrazione e lo salvo in memoria
        sign=0;
        ScriviChar(0,indCalib1); //Imposto a 0 il valore della calibrazione principle e lo salvo in memoria
        calib1=0;
        ScriviChar(0,indCalib2); //Imposto a 0 il valore della seconda calibrazione e lo salvo in memoria
        calib2=0;
        ScriviChar(0,indCalib3); //Imposto a 0 il valore della terza calibrazione e lo salvo in memoria
        calib3=0;
        ScriviChar(0,indEnFx); //Disabilito gli effetti del display e ne salvo lo stato in memoria
        enFx=0;
    }

    enDp=1; //Abilito il punto decimale

    while (1)   //Ciclo infinito
    {
        if(enFx==1 && clkEnFx==1) //Se gli effetti display sono abilitati ed  trascorso un minuto
        {
            generaEffetto(); //Attiva un effetto del display
            clkEnFx=0; //Azzera il flag di abilitazione ad ogni minuto
        }

        visualizzaOra(ore,min); //Esegui un ciclo di visualizzazione dell'ora

        if(S1==0 && S4==0) //Verifica pressione contemporanea pulsanti S1 e S4
        {
            antirimbalzo(); //Antirimbalzo software

            if(S1==0 && S4==0) //Se i pulsanti sono ancora premuti
            {
                programmaCalibrazioni(); //Attiva la programmazione delle calibrazioni e dell'abilitzione degli effetti display
            }
        }

        if(S1==0 && S2==1 && S3==1 && S4==1) //Verifica pressione pulsante S1 decremento ore
        {
            antirimbalzo(); //Antirimbalzo software
            
            if(S1==0 && S2==1 && S3==1 && S4==1) //Se il pulsante  ancora premuto
            {
                if(ore!=0) //Se le ore sono diverse da 0
                {
                    ore--;  //Decrementa le ore
                }
                else    //Altrimenti
                {
                    ore=23; //Riporta le ore a 23
                }
                ms=0;   //Azzera i millisecondi
                sec=0;  //Azzera i secondi
                TMR0=133; //Reiposta il preset del Timer0
            }
        }

        if(S1==1 && S2==0 && S3==1 && S4==1) //Verifica pressione pulsante S2 incremento ore
        {
            antirimbalzo(); //Antirimbalzo software
            
            if(S1==1 && S2==0 && S3==1 && S4==1) //Se il pulsante  ancora premuto
            {
                if(ore<23)  //Se le ore sono inferiori a 23
                {
                    ore++; //Incrementa le ore
                }
                else    //Altrimenti
                {
                    ore=0;  //Riporta le ore a 0
                }
                ms=0;   //Azzera i millisecondi
                sec=0;  //Azzera i secondi
                TMR0=133; //Reiposta il preset del Timer0
            }
        }

        if(S1==1 && S2==1 && S3==0 && S4==1) //Verifica pressione pulsante S3 decremento minuti
        {
            antirimbalzo(); //Antirimbalzo software
            
            if(S1==1 && S2==1 && S3==0 && S4==1) //Se il pulsante  ancora premuto
            {
                if(min!=0) //Se i minuti sono diversi da 0
                {
                    min--; //Decrementa i minuti
                }
                else    //Altrimenti
                {
                    min=59; //Riporta i minuti a 59
                }
                ms=0;   //Azzera i millisecondi
                sec=0;  //Azzera i secondi
                TMR0=133; //Reiposta il preset del Timer0
            }
        }

        if(S1==1 && S2==1 && S3==1 && S4==0) //Verifica pressione pulsante S4 incremento minuti
        {
            antirimbalzo(); //Antirimbalzo software
            
            if(S1==1 && S2==1 && S3==1 && S4==0) //Se il pulsante  ancora premuto
            {
                if(min<59)  //Se i minuti sono inferiori a 59
                {
                    min++;  //Incrementa i minuti
                }
                else    //Altrimenti
                {
                    min=0;  //Riporta a 0 i minuti
                }

                ms=0;   //Azzera i millisecondi
                sec=0;  //Azzera i secondi
                TMR0=133; //Reiposta il preset del Timer0
            }
        }        
    }
    return 1;
 }

//Funzione di interrupt principale (cuore dell'orologio)
void __interrupt timerClock(void)
{
    if(ms==500) //Se sono trascorsi 500ms (mezzo secondo)
    {
        dp=0; //Spegni il punto decimale
    }

    if(calib3Count<100) //Verifica contatore per applicazione della terza calibrazione
    {
        if(calib2Count<100) //Verifica contatore per applicazione della seconda calibrazione
        {
            if(sign==0) //Se il segno  positivo
            {
                if(ms<(999+calib1)) //Se i millisecondi sono inferiori a 999 + la calibrazione principale
                {
                    ms++; //Incrementa i millisecondi
                }
                else //Altrimenti
                {
                    ms=0; //Azzera i millisecondi

                    if(enDp==1) //Se il punto decimale  abilitato
                    {
                        dp=1;   //Riaccendi il punto decimale
                    }

                    sec++;  //Incrementa i secondi
                    calib2Count++; //Incrementa il contatore per l'applicazione della seconda calibrazione
                }
            }
            else //Altrimenti, se il segno della calibrazione  negativo
            {
                if(ms<(999-calib1)) //Se i millisecondi sono inferiori a 999 - la calibrazione principale
                {
                    ms++; //Incrementa i millisecondi
                }
                else //Altrimenti
                {
                    ms=0; //Azzera i millisecondi

                    if(enDp==1) //Se il punto decimale  abilitato
                    {
                        dp=1;   //Riaccendi il punto decimale
                    }

                    sec++;  //Incrementa i secondi
                    calib2Count++; //Incrementa il contatore per l'applicazione della seconda calibrazione
                }
            }
        }
        else //Se il contatore per l'applicazione della seconda calibrazione  arrivato a 100
        {
            if(sign==0) //Se il segno  positivo
            {
                if(ms<(999+(calib1+calib2))) //Se i millisecondi sono inferiori a 999 + la somma fra la calibrazione principale e la seconda calibrazione
                {
                    ms++; //Incrementa i millisecondi
                }
                else //Altrimenti
                {
                    ms=0; //Azzera i millisecondi

                    if(enDp==1) //Se il punto decimale  abilitato
                    {
                        dp=1;   //Riaccendi il punto decimale
                    }

                    sec++;  //Incrementa i secondi
                    calib2Count=0; //Azzera il contatore per l'applicazione della seconda calibrazione
                    calib3Count++; //Incrementa il contatore per l'applicazione della terza calibrazione
                }
            }
            else //Altrimenti, se il segno della calibrazione  negativo
            {
                if(ms<(999-(calib1+calib2))) //Se i millisecondi sono inferiori a 999 - la somma fra la calibrazione principale e la seconda calibrazione
                {
                    ms++; //Incrementa i millisecondi
                }
                else //Altrimenti
                {
                    ms=0; //Azzera i millisecondi

                    if(enDp==1) //Se il punto decimale  abilitato
                    {
                        dp=1;   //Riaccendi il punto decimale
                    }

                    sec++;  //Incrementa i secondi
                    calib2Count=0; //Azzera il contatore per l'applicazione della seconda calibrazione
                    calib3Count++; //Incrementa il contatore per l'applicazione della terza calibrazione
                }
            }
        }
    }
    else //Se il contatore per l'applicazione della terza calibrazione  arrivato a 100
    {
        if(sign==0) //Se il segno  positivo
        {
            if(ms<(999+(calib1+calib2+calib3))) //Se i millisecondi sono inferiori a 999 + la somma delle tre calibrazioni
            {
                ms++; //Incrementa i millisecondi
            }
            else //Altrimenti
            {
                ms=0; //Azzera i millisecondi

                if(enDp==1) //Se il punto decimale  abilitato
                {
                    dp=1;   //Riaccendi il punto decimale
                }

                sec++;  //Incrementa i secondi
                calib2Count=0; //Azzera il contatore per l'applicazione della seconda calibrazione
                calib3Count=0; //Azzera il contatore per l'applicazione della terza calibrazione
            }
        }
        else //Altrimenti, se il segno della calibrazione  negativo
        {
            if(ms<(999-(calib1+calib2+calib3))) //Se i millisecondi sono inferiori a 999 - la somma delle tre calibrazioni
            {
                ms++; //Incrementa i millisecondi
            }
            else //Altrimenti
            {
                ms=0; //Azzera i millisecondi

                if(enDp==1) //Se il punto decimale  abilitato
                {
                    dp=1;   //Riaccendi il punto decimale
                }

                sec++;  //Incrementa i secondi
                calib2Count=0; //Azzera il contatore per l'applicazione della seconda calibrazione
                calib3Count=0; //Azzera il contatore per l'applicazione della terza calibrazione
            }
        }
    }


    if(sec>59) //Se i secondi sono arrivati a 60
    {
        sec=0;  //Azzera i secondi
        min++;  //Incrementa i minuti
        clkEnFx=1; //Imposta il flag a 1 per abilitare la visualizzazione di un effetto
    }

    if(min>59) //Se i minuti sono arrivati a 60
    {
        min=0;  //Azzera i minuti
        ore++;  //Incrementa le ore
    }

    if(ore>23) //Se le ore sono arrivate a 24
    {
        ore=0;  //Azzera le ore
    }

    TMR0=133;   //Reimposta il conteggio del Timer 0 (131+2 cicli)
    INTCONbits.TMR0IF=0; //Riabilita interrupt
    
    return;
}


void antirimbalzo()
//Funzione di pausa utilizzata nella lettura dello stato di un pulsante per discriminare la pressione del tasto stesso da un disturbo. Durante
//la pausa vengono eseguiti costantemente dei cicli di visualizzazione dell'ora per evitare sfarfallii dei diaplay
{
    unsigned int conteggio; //Variabile per la ripetizione del ciclo (equivalente alla durata dell'antirimbalzo)
    for(conteggio=0;conteggio<15;conteggio++)  //Ciclo for che si ripete per 15 volte
    {
        visualizzaOra(ore,min);    //Esegui un ciclo di visualizzazione dell'ora
    }
    return;
}