
/***********************************************
*           ***  PROGRAMMAZIONE  ***           *
*        Libreria per la programmazione        *
*       dell'orologio UP con decoder 4511      *
*                                              *
*   Firmware version: 2.0                      *
*   Autore: Battistella Mirco                  *
*   Data: 09/11/2013                           *
*   Piattaforma software: MPLAB X v1.80        *
*   Compilatore: XC8 v1.20                     *
*                                              *
***********************************************/


#include <xc.h>
#include "Decoder_4511.h" //File per il dialogo con la memoria eeprom del pic
#include "eepromX_PIC16F1827.h" //File per il comando del decoder 4511

#ifndef PROGRAMMAZIONE_H //Se la libreria non  gi stata inclusa
#define	PROGRAMMAZIONE_H //Imposta un #define per evitare doppie inclusioni

//Definizione pulsanti di comando (se non sono gi stati definiti)
#ifndef S1
#define S1 PORTAbits.RA4  // H-
#endif
#ifndef S2
#define S2 PORTAbits.RA3  // H+
#endif
#ifndef S3
#define S3 PORTAbits.RA2  // M-
#endif
#ifndef S4
#define S4 PORTAbits.RA1  // M+
#endif

visualizzaProgrammazione(unsigned char param)
//Funzione per la visualizzazione dei paramentri in fase di programmazione sui quattro display
{
    unsigned char cifra1=0; //Varibile con il valore da visualizzare nel primo display
    unsigned char cifra2=0; //Varibile con il valore da visualizzare nel secondo display
    unsigned char cifra3=0; //Varibile con il valore da visualizzare nel terzo display
    unsigned char cifra4=0; //Varibile con il valore da visualizzare nel quarto display

    if(param==0) //Se il parametro da visualizzare  il primo (segno della calibrazione)
    {
        cifra1=5; //Assegna il valore 5 ("S") alla variabile 1
        cifra2=10; //Assegna il valore 10 (nessuna cifra) alla variabile 2
        cifra3=10; //Assegna il valore 10 (nessuna cifra) alla variabile 3
        cifra4=sign; //Assegna il valore del segno della calibrazione (0=positivo / 1=negativo) alla variabile 4
    }
    else if(param==1) //Se il parametro da visualizzare  il secondo (calibrazione principale)
    {
        cifra1=10; //Assegna il valore 10 (nessuna cifra) alla variabile 1
        cifra2=calib1/100; //Assegna le centinaia della calibrazione principale alla variabile 2
        cifra3=(calib1%100)/10; //Assegna le decine della calibrazione principale alla variabile 3
        cifra4=calib1%10; //Assegna le unit della calibrazione principale alla variabile 4
    }
    else if(param==2) //Se il parametro da visualizzare  il terzo (seconda calibrazione)
    {
        cifra1=2; //Assegna il valore 2 (parametro n. 2) alla variabile 1
        cifra2=10; //Assegna il valore 10 (nessuna cifra) alla variabile 2
        cifra3=calib2/10; //Assegna le decine della seconda calibrazione alla variabile 3
        cifra4=calib2%10; //Assegna le unit della seconda calibrazione alla variabile 4
    }
    else if(param==3) //Se il parametro da visualizzare  il quarto (terza calibrazione)
    {
        cifra1=3; //Assegna il valore 3 (parametro n. 3) alla variabile 1
        cifra2=10; //Assegna il valore 10 (nessuna cifra) alla variabile 2
        cifra3=calib3/10; //Assegna le decine della terza calibrazione alla variabile 3
        cifra4=calib3%10; //Assegna le unit della terza calibrazione alla variabile 4
    }
    else //Altrimenti, se il parametro da visualizzare  il quinto (abilitazione effetti display)
    {
        cifra1=4; //Assegna il valore 4 (parametro n. 4) alla variabile 1
        cifra2=10; //Assegna il valore 10 (nessuna cifra) alla variabile 2
        cifra3=10; //Assegna il valore 10 (nessuna cifra) alla variabile 3
        cifra4=enFx; //Assegna il valore del flag di abilitazione degli effetti display (0=disattivi / 1=attivi) alla variabile 4
    }

    visualizzaSingleDigit(cifra1, cifra2, cifra3, cifra4); //Esegui un ciclo di visualizzazione della programmazione sul display
    
    return;
}

void antirimbalzoP(unsigned char param)
//Funzione di pausa utilizzata nella lettura dello stato di un pulsante per discriminare la pressione del tasto stesso da un disturbo. Durante
//la pausa vengono eseguiti costantemente dei cicli di visualizzazione del parametro in corso di programmazione per evitare sfarfallii dei diaplay
{
    unsigned int conteggio; //Variabile per la ripetizione del ciclo (equivalente alla durata dell'antirimbalzo)
    for(conteggio=0;conteggio<15;conteggio++)  //Ciclo for che si ripete per 15 volte
    {
        visualizzaProgrammazione(param); //Esegue un ciclo di visualizzazione della programmazione
    }
    
    return;
}

void programSign()
//Funzione per la programmazione del segno della calibrazione
{
    unsigned char prosegui=1; //Flag per l'uscita dal ciclo while

    while(prosegui) //Ripeti il ciclo fino a quando il flag non viene azzerato
    {
        visualizzaProgrammazione(0); //Esegui un ciclo di visualizzazione della programamzione corrente

        if(S4==0) //Verifica pressione pulsante S4 
        {
            antirimbalzoP(0); //Antirimbalzo software

            if(S4==0) //Se il pulsante  ancora premuto
            {
                sign=0; //Setta il segno della calibrazione a positivo
            }
        }

        if(S3==0) //Verifica pressione pulsante S3
        {
            antirimbalzoP(0); //Antirimbalzo software

            if(S3==0) //Se il pulsante  ancora premuto
            {
                sign=1; //Setta il segno della calibrazione a negativo
            }
        }

        if(S1==0) //Verifica pressione pulsante S1
        {
            antirimbalzoP(0); //Antirimbalzo software

            if(S1==0) //Se il pulsante  ancora premuto
            {
                prosegui=0; //Azzera il flag per uscire dal ciclo while
                ScriviChar(sign,indSign); //Salva il valore del segno in memoria
            }
        }
    }
    
    return;
}

void programCalib1()
//Funzione per la programmazione della calibrazione principale
{
    unsigned char prosegui=1; //Flag per l'uscita dal ciclo while
    
    while(prosegui) //Ripeti il ciclo fino a quando il flag non viene azzerato
    {
        visualizzaProgrammazione(1); //Esegui un ciclo di visualizzazione della programamzione corrente

        if(S4==0) //Verifica pressione pulsante S4
        {
            antirimbalzoP(1); //Antirimbalzo software

            if(S4==0) //Se il pulsante  ancora premuto
            {
                if(calib1<254) //Se il valore della calibrazione  inferiore a 254
                {
                    calib1++; //Incrementa il valore della calibrazione
                }
                else //Altrimenti
                {
                    calib1=0; //Porta a 0 il valore della calibrazione
                }
            }
        }

        if(S3==0) //Verifica pressione pulsante S3
        {
            antirimbalzoP(1); //Antirimbalzo software

            if(S3==0) //Se il pulsante  ancora premuto
            {
                if(calib1>1) //Se il valore della calibrazione  maggiore di 1
                {
                    calib1--; //Decrementa il valore della calibrazione
                }
                else //Altrimenti
                {
                    calib1=254; //Porta a 254 il valore della calibrazione
                }
            }
        }

        if(S1==0) //Verifica pressione pulsante S1
        {
            antirimbalzoP(1); //Antirimbalzo software

            if(S1==0) //Se il pulsante  ancora premuto
            {
                prosegui=0; //Azzera il flag per uscire dal ciclo while
                ScriviChar(calib1,indCalib1); //Salva il valore della calibrazione principale in memoria
            }
        }
    }
    
    return;
}

void programCalib2()
//Funzione per la programmazione della seconda calibrazione
{
    unsigned char prosegui=1; //Flag per l'uscita dal ciclo while
    
    while(prosegui) //Ripeti il ciclo fino a quando il flag non viene azzerato
    {
        visualizzaProgrammazione(2); //Esegui un ciclo di visualizzazione della programamzione corrente

        if(S4==0) //Verifica pressione pulsante S4
        {
            antirimbalzoP(2); //Antirimbalzo software

            if(S4==0) //Se il pulsante  ancora premuto
            {
                if(calib2<99) //Se il valore della calibrazione  inferiore a 99
                {
                    calib2++; //Incrementa il valore della calibrazione
                }
                else //Altrimenti
                {
                    calib2=0; //Porta a 0 il valore della calibrazione
                }
            }
        }

        if(S3==0) //Verifica pressione pulsante S3
        {
            antirimbalzoP(2); //Antirimbalzo software

            if(S3==0) //Se il pulsante  ancora premuto
            {
                if(calib2>1) //Se il valore della calibrazione  maggiore di 1
                {
                    calib2--; //Decrementa il valore della calibrazione
                }
                else //Altrimenti
                {
                    calib2=99; //Porta a 99 il valore della calibrazione
                }
            }
        }

        if(S1==0) //Verifica pressione pulsante S1
        {
            antirimbalzoP(2); //Antirimbalzo software

            if(S1==0) //Se il pulsante  ancora premuto
            {
                prosegui=0; //Azzera il flag per uscire dal ciclo while
                ScriviChar(calib2,indCalib2); //Salva il valore della seconda calibrazione in memoria
            }
        }
    }
    
    return;
}

void programCalib3()
//Funzione per la programmazione della terza calibrazione
{
    unsigned char prosegui=1; //Flag per l'uscita dal ciclo while
    
    while(prosegui) //Ripeti il ciclo fino a quando il flag non viene azzerato
    {
        visualizzaProgrammazione(3); //Esegui un ciclo di visualizzazione della programamzione corrente

        if(S4==0) //Verifica pressione pulsante S4
        {
            antirimbalzoP(3); //Antirimbalzo software

            if(S4==0) //Se il pulsante  ancora premuto
            {
                if(calib3<99) //Se il valore della calibrazione  minore di 99
                {
                    calib3++; //Incrementa il valore della calibrazione
                }
                else //Altrimenti
                {
                    calib3=0; //Porta a 0 il valore della calibrazione
                }
            }
        }

        if(S3==0) //Verifica pressione pulsante S3
        {
            antirimbalzoP(3); //Antirimbalzo software

            if(S3==0) //Se il pulsante  ancora premuto
            {
                if(calib3>1) //Se il valore della calibrazione  maggiore di 1
                {
                    calib3--; //Decrementa il valore della calibrazione
                }
                else //Altrimenti
                {
                    calib3=99; //Porta a 99 il valore della calibrazione
                }
            }
        }

        if(S1==0) //Verifica pressione pulsante S1
        {
            antirimbalzoP(3); //Antirimbalzo software

            if(S1==0) //Se il pulsante  ancora premuto
            {
                prosegui=0; //Azzera il flag per uscire dal ciclo while
                ScriviChar(calib3,indCalib3); //Salva il valore della terza calibrazione in memoria
            }
        }
    }
    
    return;
}

void programEnFx()
//Funzione per la programmazione del flag di abilitazione degli effetti display
{
    unsigned char prosegui=1; //Flag per l'uscita dal ciclo while

    while(prosegui) //Ripeti il ciclo fino a quando il flag non viene azzerato
    {
        visualizzaProgrammazione(4); //Esegui un ciclo di visualizzazione della programamzione corrente

        if(S4==0) //Verifica pressione pulsante S4
        {
            antirimbalzoP(4); //Antirimbalzo software

            if(S4==0) //Se il pulsante  ancora premuto
            {
                enFx=0; //Disabilita effetti del display
            }
        }

        if(S3==0) //Verifica pressione pulsante S3
        {
            antirimbalzoP(4); //Antirimbalzo software

            if(S3==0) //Se il pulsante  ancora premuto
            {
                enFx=1; //Abilita effetti del display
            }
        }

        if(S1==0) //Verifica pressione pulsante S1
        {
            antirimbalzoP(4); //Antirimbalzo software

            if(S1==0) //Se il pulsante  ancora premuto
            {
                prosegui=0; //Azzera il flag per uscire dal ciclo while
                ScriviChar(enFx,indEnFx); //Salva lo stato di abilitazione degli effetti in memoria
            }
        }
    }
    
    return;
}

void nPress(unsigned char param) //Funzione per verificare che non ci siano tasti premuti (per evitare impostazioni indesiderate)
{
    while(S1==0 || S2==0 || S3==0 || S4==0) //Esegui il ciclo fino a quando non ci sono pi tasti premuti
    {
        visualizzaProgrammazione(param); //Esegui un ciclo di visualizzazione della programamzione corrente
    }

    return;
}
void programmaCalibrazioni()
//Funzione principale di programmazione che racchiude tutte le sotto-programmazioni dei singoli parametri
{
    INTCONbits.GIE=0; //Disabilita gli interrupt (per precauzione)
    dp=0; //Disabilita il punto decimale
    nPress(0); //Verifica che non ci siano tasti premuti
    programSign(); //Attiva la programmazione del segno della calibrazione
    nPress(0); //Verifica che non ci siano tasti premuti
    programCalib1(); //Attiva la programmazione della calibrazione prinicpale
    nPress(1); //Verifica che non ci siano tasti premuti
    programCalib2(); //Attiva la programmazione della seconda calibrazione
    nPress(2); //Verifica che non ci siano tasti premuti
    programCalib3(); //Attiva la programmazione della terza calibrazione
    nPress(3); //Verifica che non ci siano tasti premuti
    programEnFx(); //Attiva la programmazione del flag di abilitazione degli effetti display
    nPress(4); //Verifica che non ci siano tasti premuti
    ore=0; //Azzera le ore
    min=0; //Azzera i minuti
    sec=0; //Azzera i secondi
    ms=0; //Azzera i millisecondi
    TMR0=131; //Reimposta il preset del Timer0
    INTCONbits.GIE=1; //Riabilita gli interrupt
}

#endif	/* PROGRAMMAZIONE_H */