
/***********************************************
*             ***  EEPROM X  ***               *
* Libreria lettura/scrittura eeprom PIC16F1827 *
*                                              *
*   Firmware version: 2.0                      *
*   Autore: Battistella Mirco                  *
*   Data: 09/11/2013                           *
*   Piattaforma software: MPLAB X v1.80        *
*   Compilatore: XC8 v1.20                     *
*                                              *
***********************************************/


#include <xc.h>

#ifndef EEPROMX1827_H
#define EEPROMX1827_H

unsigned char LeggiChar(unsigned char indirizzo)
//Funzione per la lettura di un byte contenuto nella memoria eeprom interna
{
    unsigned char valoreLetto; //Variabile di ritorno

    EEADRL=indirizzo; //Imposto nel registro EEADRL l'indirizzo della memoria da cui voglio leggere
    EECON1bits.CFGS=0; //Seleziono l'accesso alla program flash o alla data eeprom
    EECON1bits.EEPGD=0; //Seleziono l'accesso alla data eeprom
    EECON1bits.RD=1; //Avvio il ciclo di lettura
    valoreLetto=EEDATL; //Assegno il valore letto alla variabile di ritorno

    return valoreLetto; //Assegno alla funzione il valore letto
}


void ScriviChar(unsigned char dato, unsigned char indirizzo)
//Funzione per la scrittura di un byte nella memoria eeprom interna
{
    unsigned char interruptAttivi=0; //Variabile per la memorizzazione dello stato degli interrupt (0=non attivi / 1=attivi)
    EEADRL=indirizzo; //Imposto nel registro EEADRL l'indirizzo nel quale voglio memorizzare il dato
    EEDATL=dato; //Imposto nel registro EEDATAL il dato da memorizzare
    EECON1bits.CFGS=0; //Seleziono l'accesso alla program flash o alla data eeprom
    EECON1bits.EEPGD=0; //Seleziono l'accesso alla data eeprom
    EECON1bits.WREN=1; //Abilito la scrittura sull'eeprom

    if(INTCONbits.GIE) //Se gli interrupt sono attivi
    {
        interruptAttivi=1; //ne memorizzo lo stato di "attivato"
        INTCONbits.GIE=0; //e disabilito gli interrupt (per evitare problemi durante la scrittura)
    }

    EECON2=0x55; //Scrivo il numero 0x55 sul registro EECON2 (vedi datasheet)
    EECON2=0xAA; //Scrivo il numero 0xAA sul registro EECON2 (vedi datasheet)

    EECON1bits.WR=1; //Avvio il ciclo di scrittura

    while(EECON1bits.WR==1); //Verifico che la scrittura sia completata prima di eseguire altre operazioni
    EECON1bits.WREN=0; //Disabilito la scrittura sull'eeprom

    if(interruptAttivi) //Se gli interrupt erano attivi
    {
        INTCONbits.GIE=1; //riattivo gli interrupt
    }

    return;
}

unsigned char ScriviCharCV(unsigned char dato, unsigned char indirizzo)
//Funzione per la scrittura di un byte nella memoria eeprom interna con verifica del dato salvato
{
    unsigned char interruptAttivi=0; //Variabile per la memorizzazione dello stato degli interrupt (0=non attivi / 1=attivi)
    EEADRL=indirizzo; //Imposto nel registro EEADRL l'indirizzo nel quale voglio memorizzare il dato
    EEDATL=dato; //Imposto nel registro EEDATAL il dato da memorizzare
    EECON1bits.CFGS=0; //Seleziono l'accesso alla program flash o alla data eeprom
    EECON1bits.EEPGD=0; //Seleziono l'accesso alla data eeprom
    EECON1bits.WREN=1; //Abilito la scrittura sull'eeprom

    if(INTCONbits.GIE) //Se gli interrupt sono attivi
    {
        interruptAttivi=1; //ne memorizzo lo stato di "attivato"
        INTCONbits.GIE=0; //e disabilito gli interrupt (per evitare problemi durante la scrittura)
    }

    EECON2=0x55; //Scrivo il numero 0x55 sul registro EECON2 (vedi datasheet)
    EECON2=0xAA; //Scrivo il numero 0xAA sul registro EECON2 (vedi datasheet)

    EECON1bits.WR=1; //Avvio il ciclo di scrittura

    while(EECON1bits.WR==1); //Verifico che la scrittura sia completata prima di eseguire altre operazioni
    EECON1bits.WREN=0; //Disabilito la scrittura sull'eeprom

    if(interruptAttivi) //Se gli interrupt erano attivi
    {
        INTCONbits.GIE=1; //riattivo gli interrupt
    }

    //Eseguo un ciclo di lettura per confermare che il dato sia stato scritto correttamente
    if(LeggiChar(indirizzo)==dato) //Se il dato letto coincide con quello che ho precedentemente scritto
    {
        return 1; //Ritorna 1 (dato scritto correttamente)
    }
    else //Altrimenti
    {
        return 0; //Ritorna 0 (errore nella scrittura del dato)
    }
}
#endif	/* EEPROMX1827_H */